
#ifndef SERVER_H
#define SERVER_H

#include <string.h>
#include <strings.h>
#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>

#include "param.h"

#define MAX_FD 256 
/* #define MAX_FD FD_SETSIZE */

/* important relationships in Fd_State:
    * top 3 states must be (in order): _connected _output _output_then_die
    * _pre_backend, _no_backend, _ready_for_query must all be <
	    _requested_backend
*/
typedef enum
{
    fs_pre_backend, fs_no_backend, fs_ready_for_query, fs_requested_backend,
    fs_pending_connect, fs_connected, fs_output, fs_output_then_die
} Fd_State;

/* important relationships in Fd_Status:
    * _client_keepalive is < _client, and the only thing < _client
*/
typedef enum
{
    ft_client_keepalive, ft_client, ft_backend, ft_other
} Fd_Status;

typedef struct
{
    char *buffer;
    int buffer_len;
    int pos[MAX_FD];
    int num_chars[MAX_FD];
    int next_busy[MAX_FD];
    int prev_busy[MAX_FD];
    int first_busy, last_busy;
} Buffer;

typedef struct
{
    int port;
    int listen_fd;
    int parent_fd;
    int max_connection_fd;
    int num_connections;
    int max_connections;
    int busy;
    int fd_partner[MAX_FD];
    Fd_State fd_state[MAX_FD];
    Fd_Status fd_status[MAX_FD];
    unsigned long fd_connection_ip[MAX_FD];
    short fd_connection_port[MAX_FD];
    int content_length[MAX_FD];
    int control_fd;
    int control_port;
    fd_set read_fds;
    fd_set write_fds;
    fd_set except_fds;
    Buffer writebuf;
} Server;

#define DONT_CARE 0

Server *server_new(unsigned long ip, int port, int listen_queue);
void set_select_timeout(struct timeval *timeout);
void server_main_loop(Server *me, void (*write_fn)(Server *, int),
	void (*read_fn)(Server *, int), void (*except_fn)(Server *, int));
void shutdown_ipc(Server *serv);

void set_server_read_fd(Server *serv, int fd);
void reset_server_read_fd(Server *serv, int fd);
void set_server_write_fd(Server *serv, int fd);
void reset_server_write_fd(Server *serv, int fd);

void change_state(Server *serv, int fd, Fd_State new_state);

int locate_space(Server *serv, int *start, int *end);
int generic_write(int fd, Server *serv, int chars_to_write);

int add_data(Server *serv, int fd, char *buffer, int start, int length,
	Fd_State data_type);
void move_data(Server *serv, int from_fd, int to_fd, int length);
void swap_data(Server *serv, int fd1, int fd2);
void delete_data(Server *serv, int fd);
void append_data(Server *serv, int fd, int *start, int *end,
	Fd_State data_type);

char *grow_buffer(Server *serv, int *free_start, int *free_end, int we_free);

void backdate_read(Server *serv, int fd, int *chars_read, int *start);

#endif /* SERVER_H */
